import * as i0 from '@angular/core';
import { InjectionToken, inject, Injectable, Optional, Inject, NgModule, SkipSelf } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { DateTime } from 'luxon';
import { of } from 'rxjs';
import { NgxLibServiceBase } from '@cleavelandprice/ngx-lib';
import { CommonModule } from '@angular/common';

const ActiveDirectoryServiceOptionsToken = new InjectionToken('');

class ActiveDirectoryService extends NgxLibServiceBase {
    #services = {
        http: inject(HttpClient)
    };
    #cache;
    #defaultCacheLifeMinutes = 720;
    //#region Properties
    get #cacheExpired() {
        if (!this.#cache) {
            return true;
        }
        const expiration = this.#cache.lastUpdate.plus({ minutes: this.options.caching.lifeMinutes });
        return DateTime.now() > expiration;
    }
    //#endregion
    constructor(options) {
        super(options);
        this.options = options;
        this.options = {
            ...options,
            caching: options?.caching ?? {
                enabled: false,
                lifeMinutes: this.#defaultCacheLifeMinutes
            },
            logging: options?.logging ?? {
                dataCached: false,
                cachedDataReturned: false,
                freshDataReturned: false,
                cacheCleared: false
            }
        };
    }
    getUsers(includePhoto = false, includeDisabled = false, parentOU = '') {
        // If caching is enabled, there will be two calls to the API if the cache is null/expired
        // One call is to update the cache and the other is to return the data that the user requested
        // Subsequent calls will bypass the API and returned cached data until it expires
        const observable$ = this.#services.http.get(`${this.apiUrl}/users/${includePhoto}/${includeDisabled}/${parentOU}`);
        if (this.options.caching.enabled) {
            if (this.#cacheExpired) {
                // Get the data and cache it
                observable$.subscribe(data => {
                    this.#cache = {
                        data,
                        lastUpdate: DateTime.now()
                    };
                    if (this.options.logging.dataCached) {
                        console.log(`Data cached for ${this.options.caching.lifeMinutes} minutes`);
                    }
                });
            }
            else {
                // Cache exists and hasn't expired yet
                if (this.options.logging.cachedDataReturned) {
                    console.log('Returning cached users');
                }
                return of(this.#cache.data);
            }
        }
        // Cache is either disabled or expired, return data from API
        if (this.options.logging.freshDataReturned) {
            console.log('Returning fresh users');
        }
        return observable$;
    }
    getUser(accountName, includePhoto = false) {
        return this.#services.http.get(`${this.apiUrl}/users/name/${accountName}/${includePhoto}`);
    }
    clearCache() {
        this.#cache = undefined;
        if (this.options.logging.cacheCleared) {
            console.log('Cache cleared');
        }
    }
    getDomains() {
        return this.#services.http.get(`${this.apiUrl}/activedirectory/domains`);
    }
    getOUs(parentOU = '') {
        return this.#services.http.get(`${this.apiUrl}/activedirectory/ous/${parentOU}`);
    }
    getContainers(parentOU = '') {
        return this.#services.http.get(`${this.apiUrl}/activedirectory/containers/${parentOU}`);
    }
    getGroups(parentOU = '') {
        return this.#services.http.get(`${this.apiUrl}/activedirectory/groups/${parentOU}`);
    }
    getDistinctUserAttributeValues(attributeName) {
        return this.#services.http.get(`${this.apiUrl}/activedirectory/distinct/${attributeName}`);
    }
    getDepartments() {
        return this.#services.http.get(`${this.apiUrl}/activedirectory/departments`);
    }
    getTitles() {
        return this.#services.http.get(`${this.apiUrl}/activedirectory/titles`);
    }
    getComputers(includeDisabled = false, parentOU = '') {
        return this.#services.http.get(`${this.apiUrl}/activedirectory/computers/${includeDisabled}/${parentOU}`);
    }
    getServers(includeDisabled = false, parentOU = '') {
        return this.#services.http.get(`${this.apiUrl}/activedirectory/servers/${includeDisabled}/${parentOU}`);
    }
    getWorkstations(includeDisabled = false, parentOU = '') {
        return this.#services.http.get(`${this.apiUrl}/activedirectory/workstations/${includeDisabled}/${parentOU}`);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.3.5", ngImport: i0, type: ActiveDirectoryService, deps: [{ token: ActiveDirectoryServiceOptionsToken, optional: true }], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "17.3.5", ngImport: i0, type: ActiveDirectoryService, providedIn: 'root' }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.3.5", ngImport: i0, type: ActiveDirectoryService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: () => [{ type: undefined, decorators: [{
                    type: Optional
                }, {
                    type: Inject,
                    args: [ActiveDirectoryServiceOptionsToken]
                }] }] });

class ActiveDirectoryModule {
    static forRoot(config) {
        return {
            ngModule: ActiveDirectoryModule,
            providers: [
                {
                    provide: ActiveDirectoryServiceOptionsToken,
                    useValue: config
                }
            ]
        };
    }
    constructor(parentModule) {
        if (parentModule) {
            throw new Error('ActiveDirectoryModule is already loaded. Import it in the AppModule only');
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.3.5", ngImport: i0, type: ActiveDirectoryModule, deps: [{ token: ActiveDirectoryModule, optional: true, skipSelf: true }], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "17.3.5", ngImport: i0, type: ActiveDirectoryModule, imports: [CommonModule] }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "17.3.5", ngImport: i0, type: ActiveDirectoryModule, imports: [CommonModule] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.3.5", ngImport: i0, type: ActiveDirectoryModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [],
                    imports: [
                        CommonModule
                    ],
                    exports: []
                }]
        }], ctorParameters: () => [{ type: ActiveDirectoryModule, decorators: [{
                    type: Optional
                }, {
                    type: SkipSelf
                }] }] });

/**
 * Generated bundle index. Do not edit.
 */

export { ActiveDirectoryModule, ActiveDirectoryService, ActiveDirectoryServiceOptionsToken };
//# sourceMappingURL=cleavelandprice-ngx-lib-active-directory.mjs.map
