import * as i0 from '@angular/core';
import { InjectionToken, inject, output, Injectable, Optional, Inject, NgModule, SkipSelf } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { JwtHelperService } from '@auth0/angular-jwt';
import { NgxLibServiceBase } from '@cleavelandprice/ngx-lib';
import { CommonModule } from '@angular/common';

const AuthenticationTokenServiceOptionsToken = new InjectionToken('');

class AuthenticationTokenService extends NgxLibServiceBase {
    //#region Fields
    #http;
    //#endregion
    //#region Properties
    get useSessionStorage() {
        return !!localStorage.getItem('storeTokenInSession');
    }
    get storage() {
        return this.useSessionStorage ?
            sessionStorage :
            localStorage;
    }
    //#endregion
    //#region Lifecycle
    constructor(options) {
        super(options);
        this.options = options;
        //#region Fields
        this.#http = inject(HttpClient);
        this.authenticatedChanged = output();
        this.authenticatingChanged = output();
        this.authenticationError = output();
        this.authenticating = false; // Flag set to true while the service is attempting to authenticate the user
        this.authenticated = false; // Flag set to true when the user is successfully authenticated
        this.options = {
            ...options,
            tokenName: 'token',
            authenticationEndpoint: '/authenticate/token',
            deferTokenValidation: options?.deferTokenValidation ?? false
        };
        if (!this.options.deferTokenValidation) {
            this.checkForExistingToken();
        }
    }
    //#endregion
    //#region Utilities
    //Authenticate the user and store the token in Local Storage
    login(username, password, adminGroup, additionalInfo) {
        const request = {
            username,
            password,
            adminGroup,
            additionalInfo,
            appName: this.options.appName
        };
        this.authenticating = true;
        this.authenticatingChanged.emit(this.authenticating);
        // Sending a 'raw' POST request so that we can tell HttpClient that we are expecting text instead of Json
        // Otherwise, it would throw an error trying to decode Json from the response
        this.#http.request('POST', `${this.apiUrl}${this.options.authenticationEndpoint}`, { body: request, responseType: 'text' })
            .subscribe({
            next: (token) => {
                if (token) {
                    this.validateUser(token);
                }
                else {
                    this.logout();
                }
            },
            error: (err) => {
                this.authenticating = false;
                this.authenticatingChanged.emit(this.authenticating);
                this.authenticationError.emit(err);
            }
        });
    }
    //Logout the user and delete the token from Local Storage
    logout() {
        if (this.options.tokenName && this.storage.getItem(this.options.tokenName)) {
            this.storage.removeItem(this.options.tokenName);
        }
        this.authenticatedUser = undefined;
        this.authenticated = false;
        this.authenticatedChanged.emit(false);
    }
    validateUser(token) {
        const jwtHelper = new JwtHelperService();
        const decoded = jwtHelper.decodeToken(token);
        if (jwtHelper.isTokenExpired(token)) {
            this.logout();
            return;
        }
        this.authenticatedUser = {
            admin: decoded.role === 'Admin',
            displayName: decoded.displayName,
            accountName: decoded.winaccountname,
            distinguishedName: decoded.distinguishedName,
            employeeNumber: decoded.employeeNumber,
            memberOf: decoded.memberOf
        };
        if (this.options.tokenName) {
            this.storage.setItem(this.options.tokenName, token);
        }
        this.authenticated = true;
        this.authenticatedChanged.emit(this.authenticated);
        this.authenticating = false;
        this.authenticatingChanged.emit(this.authenticating);
    }
    checkForExistingToken() {
        if (this.options.tokenName) {
            const token = this.storage.getItem(this.options.tokenName);
            if (token) {
                this.validateUser(token);
            }
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.3.5", ngImport: i0, type: AuthenticationTokenService, deps: [{ token: AuthenticationTokenServiceOptionsToken, optional: true }], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "17.3.5", ngImport: i0, type: AuthenticationTokenService, providedIn: 'root' }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.3.5", ngImport: i0, type: AuthenticationTokenService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: () => [{ type: undefined, decorators: [{
                    type: Optional
                }, {
                    type: Inject,
                    args: [AuthenticationTokenServiceOptionsToken]
                }] }] });

class AuthenticationTokenModule {
    static forRoot(options) {
        return {
            ngModule: AuthenticationTokenModule,
            providers: [
                {
                    provide: AuthenticationTokenServiceOptionsToken,
                    useValue: options
                }
            ]
        };
    }
    constructor(parentModule) {
        if (parentModule) {
            throw new Error('AuthenticationTokenModule is already loaded. Import it in the AppModule only');
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.3.5", ngImport: i0, type: AuthenticationTokenModule, deps: [{ token: AuthenticationTokenModule, optional: true, skipSelf: true }], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "17.3.5", ngImport: i0, type: AuthenticationTokenModule, imports: [CommonModule] }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "17.3.5", ngImport: i0, type: AuthenticationTokenModule, imports: [CommonModule] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.3.5", ngImport: i0, type: AuthenticationTokenModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [],
                    imports: [
                        CommonModule
                    ],
                    exports: []
                }]
        }], ctorParameters: () => [{ type: AuthenticationTokenModule, decorators: [{
                    type: Optional
                }, {
                    type: SkipSelf
                }] }] });

/**
 * Generated bundle index. Do not edit.
 */

export { AuthenticationTokenModule, AuthenticationTokenService, AuthenticationTokenServiceOptionsToken };
//# sourceMappingURL=cleavelandprice-ngx-lib-authentication-token.mjs.map
