import * as i0 from '@angular/core';
import { InjectionToken, inject, Injectable, Optional, Inject, Pipe, input, Component, NgModule, SkipSelf } from '@angular/core';
import { of, map } from 'rxjs';
import { HttpClient } from '@angular/common/http';
import { DateTime } from 'luxon';
import { NgxLibServiceBase } from '@cleavelandprice/ngx-lib';
import { AsyncPipe, CommonModule } from '@angular/common';
import { ActiveDirectoryService } from '@cleavelandprice/ngx-lib/active-directory';

const SharePointServiceOptionsToken = new InjectionToken('');

class SharePointService extends NgxLibServiceBase {
    #services = {
        http: inject(HttpClient)
    };
    #cache = { users: undefined, photos: undefined };
    #defaultCacheLifeMinutes = 720;
    //#region Utilities
    #cacheExpired(cache) {
        if (!cache) {
            return true;
        }
        const lifeMinutes = this.options?.caching?.lifeMinutes ?? this.#defaultCacheLifeMinutes;
        const expiration = cache.lastUpdate.plus({ minutes: lifeMinutes });
        return DateTime.now() > expiration;
    }
    //#endregion
    //#region Initialization
    constructor(options) {
        super(options);
        this.options = options;
        this.options = {
            ...options,
            credentials: options?.credentials ?? {
                username: 'viewer',
                password: 'viewer'
            },
            caching: options?.caching ?? {
                enabled: false,
                lifeMinutes: this.#defaultCacheLifeMinutes
            },
            endpoints: options?.endpoints ?? {
                users: `/sharepoint/users`,
                listItems: `/sharepoint/listitems`
            },
            photos: options?.photos ?? {
                site: 'https://sharepoint.cleavelandprice.com/digitaldisplay',
                list: 'Employee Thumbnail Photos',
                additionalPropertiesToRetrieve: ['EncodedAbsUrl']
            },
            logging: options?.logging ?? {
                dataCached: false,
                cachedDataReturned: false,
                freshDataReturned: false,
                cacheCleared: false
            }
        };
    }
    //#endregion
    getListItems(request) {
        //const apiUrl = this.options?.apiUrl;
        const endpoint = this.options?.endpoints?.listItems;
        return this.#services.http.post(`${this.apiUrl}${endpoint}`, request);
        //return this.#services.http.post<ListItem[]>(this.config!.endpoints!.listItems!, request);
    }
    getPhotos() {
        const request = {
            username: this.options?.credentials?.username,
            password: this.options?.credentials?.password,
            site: this.options?.photos?.site,
            list: this.options?.photos?.list,
            additionalPropertiesToRetrieve: this.options?.photos?.additionalPropertiesToRetrieve
        };
        const observable$ = this.getListItems(request);
        if (this.options?.caching?.enabled) {
            if (this.#cacheExpired(this.#cache?.photos)) {
                // Get the data and cache it
                observable$.subscribe(data => {
                    this.#cache.photos = {
                        data,
                        lastUpdate: DateTime.now()
                    };
                    if (this.options?.logging?.dataCached) {
                        const lifeMinutes = this.options.caching?.lifeMinutes ?? this.#defaultCacheLifeMinutes;
                        console.log(`Photos cached for ${lifeMinutes} minutes`);
                    }
                });
            }
            else {
                // Cache exists and hasn't expired yet
                if (this.options.logging?.cachedDataReturned) {
                    console.log('Returning cached photos');
                }
                return of(this.#cache.photos.data);
            }
        }
        // Cache is either disabled or expired, return data from API
        if (this.options?.logging?.freshDataReturned) {
            console.log('Returning fresh photos');
        }
        return observable$;
    }
    getUsers(request, id) {
        const endpoint = this.options?.endpoints?.users;
        const observable$ = this.#services.http.post(`${this.apiUrl}${endpoint}/${id ?? ''}`, request);
        //#region Single user
        if (id) {
            return observable$;
        }
        //#endregion
        //#region Multiple users
        if (this.options?.caching?.enabled) {
            if (this.#cacheExpired(this.#cache.users)) {
                // Get the data and cache it
                observable$.subscribe(data => {
                    this.#cache.users = {
                        data: data,
                        lastUpdate: DateTime.now()
                    };
                    if (this.options?.logging?.dataCached) {
                        const lifeMinutes = this.options.caching?.lifeMinutes ?? this.#defaultCacheLifeMinutes;
                        console.log(`Users cached for ${lifeMinutes} minutes`);
                    }
                });
            }
            else {
                // Cache exists and hasn't expired yet
                if (this.options.logging?.cachedDataReturned) {
                    console.log('Returning cached users');
                }
                return of(this.#cache.users.data);
            }
        }
        // Cache is either disabled or expired, return data from API
        if (this.options?.logging?.freshDataReturned) {
            console.log('Returning fresh users');
        }
        return observable$;
        //#endregion
    }
    clearCache() {
        this.#cache = { users: undefined, photos: undefined };
        if (this.options?.logging?.cacheCleared) {
            console.log('Cache cleared');
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.3.5", ngImport: i0, type: SharePointService, deps: [{ token: SharePointServiceOptionsToken, optional: true }], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "17.3.5", ngImport: i0, type: SharePointService, providedIn: 'root' }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.3.5", ngImport: i0, type: SharePointService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: () => [{ type: undefined, decorators: [{
                    type: Optional
                }, {
                    type: Inject,
                    args: [SharePointServiceOptionsToken]
                }] }] });

class UserPhotoUrlPipe {
    #sharePointService = inject(SharePointService);
    transform(user, userPhotos) {
        const observable$ = userPhotos ? of(userPhotos) : this.#sharePointService.getPhotos();
        return observable$
            .pipe(map(photos => photos.find(pic => 
        // Try to match by employeeNumber, accountName, or title
        (pic.employeeNumber && user.employeeNumber && +pic.employeeNumber === +user.employeeNumber) ||
            (pic.accountName && pic.accountName.toLowerCase() === user.sAMAccountName.toLowerCase()) ||
            (pic.title && pic.title.toLowerCase() === user.displayName.toLowerCase()))?.encodedAbsUrl));
        /*const photo = photos
          .find(pic =>
            (pic.employeeNumber && user.employeeNumber && +pic.employeeNumber === +user.employeeNumber) ||
            (pic.accountName && pic.accountName.toLowerCase() === user.sAMAccountName.toLowerCase()) ||
            (pic.title && pic.title.toLowerCase() === user.displayName.toLowerCase()));
    
        if (!photo) {
          console.log(`No photo match for ${user.displayName}`);
        }
    
        return photo ? photo.encodedAbsUrl : null;*/
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.3.5", ngImport: i0, type: UserPhotoUrlPipe, deps: [], target: i0.ɵɵFactoryTarget.Pipe }); }
    static { this.ɵpipe = i0.ɵɵngDeclarePipe({ minVersion: "14.0.0", version: "17.3.5", ngImport: i0, type: UserPhotoUrlPipe, isStandalone: true, name: "userPhotoUrl" }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.3.5", ngImport: i0, type: UserPhotoUrlPipe, decorators: [{
            type: Pipe,
            args: [{
                    name: 'userPhotoUrl',
                    standalone: true
                }]
        }] });

class UserPhotoComponent {
    constructor() {
        this.#activeDirectoryService = inject(ActiveDirectoryService);
        this.user = input.required();
        //employee = input<CleavelandPriceUser | undefined>();
        // Or...
        //accountName = input<string | undefined>();
        this.userPhotos = input();
        this.size = input("2em");
        this.notRound = input(false);
    }
    #activeDirectoryService;
    ngOnInit() {
        ;
        if (typeof this.user() === 'string') {
            // Developer passed in an account name instead of a user object
            // We need to do the work of getting the user object ourselves
            this.#activeDirectoryService.getUser(this.user())
                .subscribe(user => this._user = user);
        }
        else {
            this._user = this.user();
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.3.5", ngImport: i0, type: UserPhotoComponent, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "17.3.5", type: UserPhotoComponent, isStandalone: true, selector: "user-photo", inputs: { user: { classPropertyName: "user", publicName: "user", isSignal: true, isRequired: true, transformFunction: null }, userPhotos: { classPropertyName: "userPhotos", publicName: "userPhotos", isSignal: true, isRequired: false, transformFunction: null }, size: { classPropertyName: "size", publicName: "size", isSignal: true, isRequired: false, transformFunction: null }, notRound: { classPropertyName: "notRound", publicName: "notRound", isSignal: true, isRequired: false, transformFunction: null } }, ngImport: i0, template: "@if (_user) {\r\n  @if ((_user | userPhotoUrl:userPhotos() | async)) {\r\n    <img\r\n      [src]=\"_user | userPhotoUrl:userPhotos() | async\"\r\n      [style.width]=\"size()\"\r\n      [style.height]=\"size()\"\r\n      [style.border-radius]=\"notRound() ? 'initial' : '100%'\">\r\n  }\r\n}\r\n", styles: [""], dependencies: [{ kind: "pipe", type: AsyncPipe, name: "async" }, { kind: "pipe", type: UserPhotoUrlPipe, name: "userPhotoUrl" }] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.3.5", ngImport: i0, type: UserPhotoComponent, decorators: [{
            type: Component,
            args: [{ selector: 'user-photo', imports: [
                        AsyncPipe,
                        UserPhotoUrlPipe
                    ], standalone: true, template: "@if (_user) {\r\n  @if ((_user | userPhotoUrl:userPhotos() | async)) {\r\n    <img\r\n      [src]=\"_user | userPhotoUrl:userPhotos() | async\"\r\n      [style.width]=\"size()\"\r\n      [style.height]=\"size()\"\r\n      [style.border-radius]=\"notRound() ? 'initial' : '100%'\">\r\n  }\r\n}\r\n" }]
        }] });

class SharePointModule {
    static forRoot(options) {
        return {
            ngModule: SharePointModule,
            providers: [
                {
                    provide: SharePointServiceOptionsToken,
                    useValue: options
                }
            ]
        };
    }
    constructor(parentModule) {
        if (parentModule) {
            throw new Error('SharePointModule is already loaded. Import it in the AppModule only');
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.3.5", ngImport: i0, type: SharePointModule, deps: [{ token: SharePointModule, optional: true, skipSelf: true }], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "17.3.5", ngImport: i0, type: SharePointModule, imports: [CommonModule,
            UserPhotoUrlPipe,
            UserPhotoComponent], exports: [UserPhotoUrlPipe,
            UserPhotoComponent] }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "17.3.5", ngImport: i0, type: SharePointModule, imports: [CommonModule] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.3.5", ngImport: i0, type: SharePointModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [],
                    imports: [
                        CommonModule,
                        UserPhotoUrlPipe,
                        UserPhotoComponent
                    ],
                    exports: [
                        UserPhotoUrlPipe,
                        UserPhotoComponent
                    ]
                }]
        }], ctorParameters: () => [{ type: SharePointModule, decorators: [{
                    type: Optional
                }, {
                    type: SkipSelf
                }] }] });

/**
 * Generated bundle index. Do not edit.
 */

export { SharePointModule, SharePointService, SharePointServiceOptionsToken, UserPhotoComponent, UserPhotoUrlPipe };
//# sourceMappingURL=cleavelandprice-ngx-lib-sharepoint.mjs.map
